## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
from CvPythonExtensions import *
import CvUtil
import ScreenInput
import time
import PyHelpers
import re
import CvMilitaryAdvisor

PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo

# globals
gc = CyGlobalContext()
ArtFileMgr = CyArtFileMgr()
localText = CyTranslator()

def compareUnits(unit1, unit2):
	if (unit1.getUnitType() > unit2.getUnitType()):
		return -1
	elif (unit1.getUnitType() < unit2.getUnitType()):
		return 1
	elif (unit1.getLevel() > unit2.getLevel()):
		return -1
	elif (unit1.getLevel() < unit2.getLevel()):
		return 1
	elif (unit1.getExperience() > unit2.getExperience()):
		return -1
	elif (unit1.getExperience() < unit2.getExperience()):
		return 1
	else:
		return 0
	
	
	
class UnitGroup:
	def __init__(self):
		self.label = ""
		self.units = []

	def setLabel(self, label):
		self.label = label
		
	def addUnit(self, idx):
		self.units.append(idx)
	
	
	
class CombatTypeGroupHelper:
	def getGroupId(self, unit):
		return unit.getUnitCombatType()
	
	def getGroupLabel(self,groupId):
		if (groupId != -1):
			return gc.getUnitCombatInfo(groupId).getDescription()
		else:
			return localText.getText( "TXT_KEY_MILITARY_ADVISOR_DOMESTIC_UNITS", () )

	def compareGroups(self, groupId1, groupId2):
		return cmp(groupId1, groupId2)




class StackGroupHelper:
	SINGLE_STACK = -1
	
	def __init__(self):
		self.stackList = [self.SINGLE_STACK]
	
	
	def getGroupId(self, unit):
		plot = CyMap().plot(unit.getX(), unit.getY())
		if (not plot.isCity()):
			numUnits = 0
			for unitIndex in range(plot.getNumUnits()):
				if (plot.getUnit(unitIndex).getOwner() == unit.getOwner()):
					numUnits += 1
			if (numUnits == 1):
				return self.SINGLE_STACK
		
		plotIndex = CyMap().plotNum(unit.getX(), unit.getY())

		if (not plotIndex in self.stackList):
			self.stackList.append(plotIndex)
		
		return plotIndex

	
	
	def getGroupLabel(self, groupId):
		if (groupId == self.SINGLE_STACK):
			return localText.getText( "TXT_KEY_MILITARY_ADVISOR_SINGLE_STACK", () )
		plot = CyMap().plotByIndex(groupId)
		if (plot.isCity()):
			return plot.getPlotCity().getName()
		else:	
			return localText.getText("TXT_KEY_MILITARY_ADVISOR_STACK", ()) + " (" + str(plot.getX()) + ", " + str(plot.getY()) + ")"


	def compareGroups(self, groupId1, groupId2):
		if (groupId1 == self.SINGLE_STACK):
			return 1
		elif (groupId2 == self.SINGLE_STACK):
			return -1
		elif (CyMap().plotByIndex(groupId1).isCity() and not CyMap().plotByIndex(groupId2).isCity()):
			return -1
		elif (CyMap().plotByIndex(groupId2).isCity() and not CyMap().plotByIndex(groupId1).isCity()):
			return 1
		else:
			return cmp(self.getGroupLabel(groupId1), self.getGroupLabel(groupId2))



class LocationGroupHelper:
	INVALID = -1
	CITY = 0
	HOME = 1
	FRIENDLY = 2
	NEUTRAL = 3
	ENEMY = 4
	
	def getGroupId(self, unit):
		plot = CyMap().plot(unit.getX(), unit.getY())
		if (not plot.isOwned()):
			return self.NEUTRAL
		if (plot.getOwner() == unit.getOwner()):
			if (plot.isCity()):
				return self.CITY
			else:
				return self.HOME
		
		unitTeam = gc.getTeam(gc.getPlayer(unit.getOwner()).getTeam())
		plotTeam = gc.getPlayer(plot.getOwner()).getTeam()
		
		if (unitTeam.isOpenBorders(plotTeam)):
			return self.FRIENDLY
		if (unitTeam.isAtWar(plotTeam)):
			return self.ENEMY
		
		return self.INVALID
		
	
	def getGroupLabel(self,groupId):
		if (groupId == self.CITY):
			return localText.getText("TXT_KEY_MILITARY_ADVISOR_CITY_GARRISON", ())
		elif (groupId == self.HOME):
			return localText.getText("TXT_KEY_MILITARY_ADVISOR_HOME_TERRITORY", ())
		elif (groupId == self.FRIENDLY):
			return localText.getText("TXT_KEY_MILITARY_ADVISOR_FRIENDLY_TERRITORY", ())
		elif (groupId == self.NEUTRAL):
			return localText.getText("TXT_KEY_MILITARY_ADVISOR_NEUTRAL_TERRITORY", ())
		elif (groupId == self.ENEMY):
			return localText.getText("TXT_KEY_MILITARY_ADVISOR_ENEMY_TERRITORY", ())
		else:
			return "ERROR, INVALID GROUP ID (LocationGroup)"

	
	def compareGroups(self, groupId1, groupId2):
		return cmp(groupId1, groupId2)
		
		
class CvEnhancedMilitaryAdvisor(CvMilitaryAdvisor.CvMilitaryAdvisor):
	"Military Advisor"

	groupSelectionName = "MIL_groupSelection"
	unitCheckBox = "MIL_unitCheckBox"
	
	NO_GROUPS_ID = 0
	STACK_GROUP_ID = 1
	COMBAT_TYPE_GROUP_ID = 2
	LOCATION_GROUP_ID = 3
	
	numGroups = 0

	unitList = None

	def __init__(self, screenId):
		self.parent = CvMilitaryAdvisor.CvMilitaryAdvisor
		self.parent.__init__(self, screenId)

										
	# Screen construction function
	def interfaceScreen(self):
							
		# Create a new screen
		screen = self.getScreen()
		if screen.isActive():
			return
 		screen.setRenderInterfaceOnly(True);
		screen.showScreen(PopupStates.POPUPSTATE_IMMEDIATE, False)

		self.EXIT_TEXT = u"<font=4>" + localText.getText("TXT_KEY_PEDIA_SCREEN_EXIT", ()).upper() + "</font>"
		self.TITLE = u"<font=4b>" + localText.getText("TXT_KEY_ENHANCED_MILITARY_ADVISOR_TITLE", ()).upper() + "</font>"

		self.nWidgetCount = 0
	
		self.iActivePlayer = gc.getGame().getActivePlayer()
		
		self.W_SCREEN = screen.getXResolution()
		self.H_SCREEN = screen.getYResolution()
		self.Y_EXIT = self.H_SCREEN - 42
		self.Y_BOTTOM_PANEL = self.H_SCREEN - 55
		
		screen = self.getScreen()
		
		
		# Set the background and exit button, and show the screen
		screen.setDimensions(0, 0, self.W_SCREEN, self.H_SCREEN)
		screen.addDrawControl(self.BACKGROUND_ID, ArtFileMgr.getInterfaceArtInfo("SCREEN_BG_OPAQUE").getPath(), 0, 0, self.W_SCREEN, self.H_SCREEN, WidgetTypes.WIDGET_GENERAL, -1, -1 )
# 		screen.addDDSGFC(self.BACKGROUND_ID, ArtFileMgr.getInterfaceArtInfo("MAINMENU_SLIDESHOW_LOAD").getPath(), 0, 0, self.W_SCREEN, self.H_SCREEN, WidgetTypes.WIDGET_GENERAL, -1, -1 )

		self.unitBackgroundName = self.getNextWidgetName()
		screen.addPanel(self.unitBackgroundName, "", "", True, True, 0, 48, 305, self.H_SCREEN - 48, PanelStyles.PANEL_STYLE_CITY_LEFT)
		self.ordersPanelName = self.getNextWidgetName()
		screen.addPanel( self.ordersPanelName, "", "", True, True, 305, self.H_SCREEN - 204, self.W_SCREEN - 305, 204
					   , PanelStyles.PANEL_STYLE_BOTTOMBAR )
		screen.addPanel( "TopPanel", u"", u"", True, False, 0, 0, self.W_SCREEN, 55, PanelStyles.PANEL_STYLE_TOPBAR )
		screen.showWindowBackground(False)
		screen.setText(self.EXIT_ID, "Background", self.EXIT_TEXT, CvUtil.FONT_RIGHT_JUSTIFY, self.X_EXIT, self.Y_EXIT, self.Z_CONTROLS, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_CLOSE_SCREEN, -1, -1 )
		
		self.viewX = 315
		self.viewY = 60
		self.viewWidth = self.W_SCREEN - 325
		self.viewHeight = self.H_SCREEN - 270
		self.viewMargin = 20
		screen.addPanel( "", u"", "", False, False, self.viewX, self.viewY
					   , self.viewWidth, self.viewHeight, PanelStyles.PANEL_STYLE_MAIN )
		
		self.initMinimap()
# 		self.initPlotView()
		
		screen.addDropDownBoxGFC(self.groupSelectionName, 15, 8, 260, WidgetTypes.WIDGET_GENERAL, -1, -1, FontTypes.GAME_FONT)
		screen.addPullDownString(self.groupSelectionName, "Don't group units", self.NO_GROUPS_ID, self.NO_GROUPS_ID, True)
		screen.addPullDownString(self.groupSelectionName, "Group by stack", self.STACK_GROUP_ID, self.STACK_GROUP_ID, False)
		screen.addPullDownString(self.groupSelectionName, "Group by combat type", self.COMBAT_TYPE_GROUP_ID, self.COMBAT_TYPE_GROUP_ID, False)
		screen.addPullDownString(self.groupSelectionName, "Group by location", self.LOCATION_GROUP_ID, self.LOCATION_GROUP_ID, False)
		
		self.unitPanelName = self.getNextWidgetName()
		self.unitGroupName = self.getNextWidgetName()
		
		activePlayer = PyPlayer(CyGame().getActivePlayer())
 		self.unitList = activePlayer.getUnitList()
 		self.selectedUnits = []
		self.unitList.sort(compareUnits)
		
		self.groupSelectionChanged(self.NO_GROUPS_ID)
		
		self.szHeader = self.getNextWidgetName()
		screen.setText(self.szHeader, "Background", self.TITLE, CvUtil.FONT_CENTER_JUSTIFY, self.X_SCREEN, self.Y_TITLE, self.Z_CONTROLS, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)

		
		
	def initMinimap(self):
		screen = self.getScreen()
		mapWidth = self.viewWidth - 2 * self.viewMargin
		mapHeight = self.viewHeight - 2 * self.viewMargin
		
		mapHeightPref = (mapWidth * CyMap().getGridHeight()) / CyMap().getGridWidth()
		if (mapHeightPref > mapHeight):
			mapWidth = (mapHeight * CyMap().getGridWidth()) / CyMap().getGridHeight()
			horMapMargin = (self.viewWidth - mapWidth) / 2
			verMapMargin = self.viewMargin
		else:
			mapHeight = mapHeightPref
			horMapMargin = self.viewMargin
			verMapMargin = (self.viewHeight - mapHeight) / 2
					
		screen.initMinimap( self.viewX + horMapMargin, self.viewX + self.viewWidth - horMapMargin
						  , self.viewY + verMapMargin, self.viewY + self.viewHeight - verMapMargin, self.Z_CONTROLS)
		screen.updateMinimapSection(False, False)

		screen.updateMinimapColorFromMap(MinimapModeTypes.MINIMAPMODE_TERRITORY, 0.3)

		screen.setMinimapMode(MinimapModeTypes.MINIMAPMODE_MILITARY)
		
		iOldMode = CyInterface().getShowInterface()
		CyInterface().setShowInterface(InterfaceVisibility.INTERFACE_MINIMAP_ONLY)
		screen.updateMinimapVisibility()
		CyInterface().setShowInterface(iOldMode)


	
	def initPlotView(self):
		self.plotview = self.getNextWidgetName()
		#TODO maybe init with view of capital
	
	
	
		
	def unitDescription(self, unit):																									
			strengthText = u""
			if (unit.getDomainType() == DomainTypes.DOMAIN_AIR):
				if (unit.airBaseCombatStr() > 0):
					if (unit.isHurt()):
						strengthText = u"%.1f/%d%c" %(((float(unit.airBaseCombatStr() * unit.currHitPoints())) / (float(unit.maxHitPoints()))), unit.airBaseCombatStr(), CyGame().getSymbolID(FontSymbols.STRENGTH_CHAR))
					else:
						strengthText = u"%d%c" %(unit.airBaseCombatStr(), CyGame().getSymbolID(FontSymbols.STRENGTH_CHAR))
			else:
				if (unit.canFight()):
					if (unit.isHurt()):
						strengthText = u"%.1f/%d%c" %(((float(unit.baseCombatStr() * unit.currHitPoints())) / (float(unit.maxHitPoints()))), unit.baseCombatStr(), CyGame().getSymbolID(FontSymbols.STRENGTH_CHAR))
					else:
						strengthText = u"%d%c" %(unit.baseCombatStr(), CyGame().getSymbolID(FontSymbols.STRENGTH_CHAR))
			movesText = u""
			if ( (unit.movesLeft() % gc.getMOVE_DENOMINATOR()) > 0 ):
				iDenom = 1
			else:
				iDenom = 0
			iCurrMoves = ((unit.movesLeft() / gc.getMOVE_DENOMINATOR()) + iDenom )
			if (unit.baseMoves() == iCurrMoves):
				movesText = u"%d%c" %(unit.baseMoves(), CyGame().getSymbolID(FontSymbols.MOVES_CHAR) )
			else:
				movesText = u"%d/%d%c" %(iCurrMoves, unit.baseMoves(), CyGame().getSymbolID(FontSymbols.MOVES_CHAR) )
			return gc.getUnitInfo(unit.getUnitType()).getDescription() + u": " + strengthText + u" " + movesText

	
	def checkBoxName(self, idx):
		return self.unitCheckBox + str(idx)
		
		
	
	def addGroupPanel(self, label, groupIndex):
		screen = self.getScreen()
		screen.attachPanel( self.unitBackgroundName, self.unitGroupName + str(groupIndex), "", ""
						  , True, False, PanelStyles.PANEL_STYLE_DAWNTOP )
		screen.attachLabel(self.unitGroupName + str(groupIndex), self.unitGroupName + str(groupIndex) + "fill1", "<font=4> </font>")
		screen.attachLabel(self.unitGroupName + str(groupIndex), self.unitGroupName + str(groupIndex) + "fill2", "<font=3> </font>")
		screen.setLabelAt( self.unitGroupName + str(groupIndex) + "Label", self.unitGroupName + str(groupIndex)
						 , u"<font=4>" + label + "</font>", CvUtil.FONT_CENTER_JUSTIFY
						 , (305 - 17) / 2, 12, -0.1, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )
	
	
	
	
	def addUnitToList(self, unit, idx):
		screen = self.getScreen()
		screen.attachPanel(self.unitBackgroundName, self.unitPanelName + str(idx), "", "", False, False, PanelStyles.PANEL_STYLE_OUT)
		screen.attachLabel( self.unitPanelName + str(idx), self.unitPanelName + str(idx) + "fill1", "            " )
		screen.addCheckBoxGFCAt( self.unitPanelName + str(idx), self.checkBoxName(idx)
							   , gc.getUnitInfo(unit.getUnitType()).getButton()
							   , ArtFileMgr.getInterfaceArtInfo("BUTTON_HILITE_SQUARE").getPath()
							   , 1, 2, 46, 46, WidgetTypes.WIDGET_GENERAL, idx, idx, ButtonStyles.BUTTON_STYLE_LABEL)
		if (unit in self.selectedUnits):
			screen.setState(self.checkBoxName(idx), true)
		
		screen.attachPanel( self.unitPanelName + str(idx), self.unitPanelName + str(idx) + "info", ""
						  , "", True, False, PanelStyles.PANEL_STYLE_EMPTY )
		screen.attachLabel( self.unitPanelName + str(idx) + "info", self.unitPanelName + str(idx) + "description"
						  , self.unitDescription(unit) )
		screen.setLabelAt( self.unitPanelName + str(idx) + "XP"
						 , self.unitPanelName + str(idx)
						 , u"XP: %d/%d" %(unit.getExperience(), unit.experienceNeeded())
						 , CvUtil.FONT_LEFT_JUSTIFY, 51, 27, -0.1, FontTypes.SMALL_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )
		screen.attachPanel( self.unitPanelName + str(idx) + "info", self.unitPanelName + str(idx) + "dummy", ""
						  , "", False, False, PanelStyles.PANEL_STYLE_EMPTY )
		screen.attachLabel( self.unitPanelName + str(idx) + "dummy", self.unitPanelName + str(idx) + "fill2", "                " )
		screen.attachMultiListControlGFC( self.unitPanelName + str(idx) + "dummy"
										, self.unitPanelName + str(idx) + "promotions"
										, "", 1, 24, 24, TableStyles.TABLE_STYLE_STANDARD )
		index = 0
		for iPromotion in range(gc.getNumPromotionInfos()):
			if (unit.isHasPromotion(iPromotion)):
				screen.appendMultiListButton( self.unitPanelName + str(idx) + "promotions"
											, gc.getPromotionInfo(iPromotion).getButton(), 0
											, WidgetTypes.WIDGET_GENERAL, -1, -1, True )
				screen.disableMultiListButton( self.unitPanelName + str(idx) + "promotions", 0, index
											 , gc.getPromotionInfo(iPromotion).getButton() )
				index += 1
	
		
	
	
	def addUnitsToList(self, units):
		for idx in range(len(units)):
			self.addUnitToList(units[idx], idx)
		
		
		
	def handleInput (self, inputClass):
		screen = self.getScreen()
# 		screen.setText( "TopPanelDebugMsg", "TopPanel", inputClass.getFunctionName()
# 					  , CvUtil.FONT_RIGHT_JUSTIFY, 1010, 20, -10, FontTypes.SMALL_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )
		if (inputClass.getFunctionName() == self.unitCheckBox and inputClass.getNotifyCode() == NotifyCode.NOTIFY_CLICKED
															  and inputClass.getFlags() & MouseFlags.MOUSE_LBUTTONUP ): # unit selected
			self.toggleUnitCheckBox(inputClass.getData1())
# 			screen.setText( "TopPanelDebugMsg", "TopPanel", u"unit selection changed"
# 						  , CvUtil.FONT_LEFT_JUSTIFY, 1010, 20, -10, FontTypes.SMALL_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )
		if ( inputClass.getFunctionName() == self.groupSelectionName and 
			 inputClass.getNotifyCode() == NotifyCode.NOTIFY_LISTBOX_ITEM_SELECTED ): # group changed
			iIndex = screen.getSelectedPullDownID(self.groupSelectionName)
			self.groupSelectionChanged(iIndex)
# 			screen.setText( "TopPanelDebugMsg", "TopPanel", "group selection changed"
# 						  , CvUtil.FONT_RIGHT_JUSTIFY, 1010, 20, -10, FontTypes.SMALL_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )
			
		return 0

		
		
		
	def toggleUnitCheckBox(self, unitIndex):
		screen = self.getScreen()
		unit = self.unitList[unitIndex]
		if ( screen.getCheckBoxState(self.checkBoxName(unitIndex)) ):
			self.selectedUnits.append(self.unitList[unitIndex])
		else:
			self.selectedUnits.remove(self.unitList[unitIndex])
		
		self.refreshMinimap()
# 		self.updatePlotView(unit.getX(), unit.getY())
		



	def groupSelectionChanged(self, groupId):
		screen = self.getScreen()
		for groupIndex in range(self.numGroups):
			screen.deleteWidget(self.unitGroupName + str(groupIndex))
		
		if (groupId == self.NO_GROUPS_ID):
			self.addUnitsToList(self.unitList)
		elif (groupId == self.STACK_GROUP_ID):
			self.createGroup(StackGroupHelper())
		elif (groupId == self.COMBAT_TYPE_GROUP_ID):
			self.createGroup(CombatTypeGroupHelper())
		elif (groupId == self.LOCATION_GROUP_ID):
			self.createGroup(LocationGroupHelper())
		


	def createGroup(self, groupHelper):
		self.groups = {}
		for idx in range(len(self.unitList)):
			groupId = groupHelper.getGroupId(self.unitList[idx])
			if (self.groups.has_key(groupId)):
				self.groups[groupId].addUnit(idx)
			else:
				self.groups[groupId] = UnitGroup()
				self.groups[groupId].addUnit(idx)
		
		keys = self.groups.keys()
		keys.sort(groupHelper.compareGroups)
		groupIndex = 0
		for groupId in keys:
			self.addGroupPanel(groupHelper.getGroupLabel(groupId), groupIndex)
			groupIndex += 1
			units = self.groups[groupId].units
			for idx in units:
				self.addUnitToList(self.unitList[idx], idx)
				
		self.numGroups = groupIndex
		
	
	def refreshMinimap(self):
		screen = self.getScreen()
		screen.minimapClearAllFlashingTiles()
		
		iColor = gc.getInfoTypeForString("COLOR_GREEN")
		for unit in self.selectedUnits:
			screen.minimapFlashPlot(unit.getX(), unit.getY(), iColor, -1)
	
	
	def updatePlotView(self, x, y):
		screen = self.getScreen()
		screen.addPlotGraphicGFC( self.plotview, self.viewX + self.viewMargin, self.viewY + self.viewMargin
								, self.viewWidth - self.viewMargin * 2, self.viewHeight - self.viewMargin * 2
								, CyMap().plot(x, y), 800, True, WidgetTypes.WIDGET_GENERAL, -1, -1 )
				
